<?php

namespace Drupal\css_slider\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\image\Plugin\Field\FieldFormatter\ImageFormatter;
use Drupal\Core\Field\EntityReferenceFieldItemListInterface;

use Drupal\Core\Template\Attribute;
use Drupal\Core\Url;

/**
 * Plugin implementation of the 'CssSliderFormatter' formatter.
 *
 * @FieldFormatter(
 *   id = "css_slider",
 *   label = @Translation("CSS Slider"),
 *   field_types = {
 *     "image"
 *   }
 * )
 */
class CssSliderFormatter extends ImageFormatter {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'controls' => TRUE,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    // Add the optionset setting.
    // $element = $this->buildSettingsForm();
    $element = [];

    // Add the image settings.
    $element = array_merge($element, parent::settingsForm($form, $form_state));
    // We don't need the link setting.
    $element['image_link']['#access'] = FALSE;

    $element['controls'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Controls'),
      '#default_value' => $this->getSetting('controls'),
    ];

    // Add the caption setting.
    // if (!empty($this->getSettings())) {
    //   $element += $this->captionSettings($this, $this->fieldDefinition);
    // }

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary[] = $this->t('Controls: @controls', [
      '@controls' => $this->getSetting('controls') ? $this->t('On') : $this->t('Off')
    ]);
    return $summary;
  }

  protected function createSlides(EntityReferenceFieldItemListInterface $items, $langcode) {
    $slides = array_map(function ($element) {
      $id = 'slide-' . $element['#item']->getValue()['target_id'];

      return [
        'attributes' => new Attribute(['id' => $id]),
        'content' => $element
      ];
    }, parent::viewElements($items, $langcode));

    return $slides;
  }

  protected function createControls(EntityReferenceFieldItemListInterface $items): array {
    $controls = array_map(function ($entity) {

      // Looping over parent::viewElements($items, $langcode)) would be more consistend
      // Id is here
      // $id = 'slide-' . $element['#item']->getValue()['target_id'];
      $id = 'slide-' . $entity->id();

      // Label is here
      // $label = $element['#item']->getValue()['alt'];

      $title = [
        '#type' => 'html_tag',
        '#tag' => 'span',
        '#value' => $entity->label(),
        '#attributes' => ['class' => ['visually-hidden']]
      ];

      return [
        '#type' => 'link',
        '#title' => $title,
        '#url' => Url::fromUserInput('#' . $id),
        '#attributes' => ['class' => ['dot']]
      ];
    }, $items->referencedEntities());

    return $controls;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [
      '#theme' => 'css_slider',
      '#slides' => $this->createSlides($items, $langcode),
      '#attached' => ['library' => ['css_slider/css_slider']]
    ];

    if($this->getSetting('controls')) {
      $elements['#controls'] = $this->createControls($items);
      $elements['#attached']['library'][] = 'css_slider/controls';
    }

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public static function isApplicable(FieldDefinitionInterface $field_definition) {
    return parent::isApplicable($field_definition) && $field_definition->getFieldStorageDefinition()->isMultiple();
  }
}
